﻿using ProjectMotorboat.Drownings;

namespace ProjectMotorboat.CollectionGenericObjects;


public abstract class AbstractCompany
{
    protected readonly int _placeSizeWidth = 190;

    protected readonly int _placeSizeHeight = 100;

    protected readonly int _pictureWidth;

    protected readonly int _pictureHeight;

    protected ICollectionGenericObjects<DrawningBoat>? _collection = null;

    
    private int GetMaxCount => _pictureWidth * _pictureHeight / (_placeSizeWidth * _placeSizeHeight);

    public AbstractCompany(int picWidth, int picHeight, ICollectionGenericObjects<DrawningBoat> collection)
    {
        _pictureWidth = picWidth;
        _pictureHeight = picHeight;
        _collection = collection;
        _collection.MaxCount = GetMaxCount;
    }
    public static int operator +(AbstractCompany company, DrawningBoat boat)
    {
        return company._collection.Insert(boat);
    }

    public static DrawningBoat operator -(AbstractCompany company, int position)
    {
        return company._collection.Remove(position);
    }
    public DrawningBoat? GetRandomObject()
    {
        Random rnd = new();
        return _collection?.Get(rnd.Next(GetMaxCount));
    }
    public Bitmap? Show()
    {
        Bitmap bitmap = new(_pictureWidth, _pictureHeight);
        Graphics graphics = Graphics.FromImage(bitmap);
        DrawBackgound(graphics);

        SetObjectsPosition();
        for (int i = 0; i < (_collection?.Count ?? 0); ++i)
        {
            DrawningBoat? obj = _collection?.Get(i);
            obj?.DrawTransport(graphics);
        }

        return bitmap;
    }
    protected abstract void DrawBackgound(Graphics g);

    protected abstract void SetObjectsPosition();
}
